(* ::Package:: *)

(***************************************************************)
(* Define various state vectors  *)
(***************************************************************)

uold = {\[CapitalPhi][x], D\[CapitalPhi][x], Ax[x], DAx[x], A\[Omega][x], DA\[Omega][x], A\[Nu][x], DA\[Nu][x]};
utraf = {Z[x], DZ[x], \[CapitalPhi][x], D\[CapitalPhi][x], A\[Omega][x], DA\[Omega][x], A\[Nu][x], DA\[Nu][x]};

ucons = {Z[x]};
\[Sigma]Vec = {\[Sigma]\[CapitalPhi], \[Sigma]Z, \[Sigma]A\[Omega], \[Sigma]A\[Nu]};
bcVec = {g\[CapitalPhi], gZ, gA\[Omega], gA\[Nu]};


ListEM={
sd^2*\[Kappa]^2*\[CapitalPhi][x]==\[Mu]\[CapitalPhi]*(-(\[Kappa]^2*\[Omega]d^2*\[CapitalPhi][x])+Derivative[2][\[CapitalPhi]][x]),
sd^2*\[Kappa]^2*Ax[x]==-(\[Kappa]^2*\[Omega]d^2*Ax[x])+sd*\[Kappa]*(-1+\[Mu]\[CapitalPhi]^(-1))*Derivative[1][\[CapitalPhi]][x]+Derivative[2][Ax][x],
sd^2*\[Kappa]^2*A\[Omega][x]==-(\[Kappa]^2*\[Omega]d^2*A\[Omega][x])+I*sd*\[Kappa]^2*(-1+\[Mu]\[CapitalPhi]^(-1))*\[Omega]d*\[CapitalPhi][x]+Derivative[2][A\[Omega]][x],
sd^2*\[Kappa]^2*A\[Nu][x]==-(\[Kappa]^2*\[Omega]d^2*A\[Nu][x])+Derivative[2][A\[Nu]][x]
};

(***************************************************************)
(* Define rules and equations for first order reduction *)
(***************************************************************)

FirstOrderRule={};
FirstOrderRule=Append[FirstOrderRule,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(\[CapitalPhi][x]\)\)-> \[Kappa]  D\[CapitalPhi][x]];
FirstOrderRule=Append[FirstOrderRule,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(Ax[x]\)\)-> \[Kappa] DAx[x]];
FirstOrderRule=Append[FirstOrderRule,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(A\[Omega][x]\)\)-> \[Kappa] DA\[Omega][x]];
FirstOrderRule=Append[FirstOrderRule,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(A\[Nu][x]\)\)-> \[Kappa] DA\[Nu][x]];

FirstEqn={};
FirstEqn=Append[FirstEqn,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(\[CapitalPhi][x]\)\)== \[Kappa] D\[CapitalPhi][x]];
FirstEqn=Append[FirstEqn,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(Ax[x]\)\)== \[Kappa] DAx[x]];
FirstEqn=Append[FirstEqn,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(A\[Omega][x]\)\)== \[Kappa] DA\[Omega][x]];
FirstEqn=Append[FirstEqn,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(A\[Nu][x]\)\)== \[Kappa] DA\[Nu][x]];

SecEqn=ListEM//.FirstOrderRule//.\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]FirstOrderRule\)//.FirstOrderRule;

(***************************************************************)
(* Make first order reduction *)
(***************************************************************)

FullEqn=Join[FirstEqn,SecEqn]//Flatten;

RHSoldRule=Collect[(Solve[FullEqn,\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]uold\)]//Flatten),uold,FullSimplify];

(***************************************************************)
(* Derive principal symbol *)
(***************************************************************)

FullEqnsOld=Join[(ListEM//.FirstOrderRule//.\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]FirstOrderRule\)//.FirstOrderRule),FirstEqn]//Flatten;

FullRulesOld=Collect[Solve[FullEqnsOld,(\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]uold\))],uold,FullSimplify]//Flatten;
RHSFullold=(\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]uold\))/.FullRulesOld;

M=(Table[Coefficient[RHSFullold[[i]],uold[[j]]],{i,8},{j,8}]//FullSimplify);

Clear[ListEM,FirstOrderRule,FirstEqn,FullEqnsOld,FullRulesOld,RHSFullold]


(***************************************************************)
(* Change of variables *)
(***************************************************************)

(* Equations for change of variables *)
ConstraintRule={
Z[x]-> - 1/\[Kappa] \!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\(Ax[x]\)\) - I \[Omega]d A\[Omega][x] - 1 / \[Mu]\[CapitalPhi] (sd \[CapitalPhi][x])
}/.RHSoldRule//FullSimplify;

DConstraintRule=Collect[{DZ[x]-> \[Kappa]^-1 (\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\((ucons[\([1]\)] /. ConstraintRule)\)\)/.RHSoldRule)},uold,FullSimplify];

ChangeofVarRuleNewToOld=Join[ConstraintRule,DConstraintRule]//Flatten;

Clear[ConstraintRule,DConstraintRule];

(***************************************************************)
(* Similarity transformation *)
(***************************************************************)

SCoV=Table[Coefficient[(utraf//.ChangeofVarRuleNewToOld)[[i]],uold[[j]] ],{i,8},{j,8}]//FullSimplify;
SCoVinv=Inverse[SCoV]//FullSimplify;
(***************************************************************)
(* Rule to go to new variables *)
(***************************************************************)

ChangeofVarRuleOldToNew=Table[uold[[i]]->(SCoVinv.utraf)[[i]],{i,8}];

(***************************************************************)
(* Rule to go to new variables *)
(***************************************************************)

Mtraf = SCoV.M.SCoVinv//Simplify;

(* Equation of motion after the rotation *)

RHStrafRule=Table[\!\(
\*SubscriptBox[\(\[PartialD]\), \(x\)]\((utraf[\([i]\)])\)\)->(Mtraf.utraf)[[i]],{i,8}];


(* General Solution *)

Evals={-((\[Kappa] Sqrt[sd^2+\[Mu]\[CapitalPhi] \[Omega]d^2])/Sqrt[\[Mu]\[CapitalPhi]]),-\[Kappa] Sqrt[sd^2+\[Omega]d^2],-\[Kappa] Sqrt[sd^2+\[Omega]d^2],-\[Kappa] Sqrt[sd^2+\[Omega]d^2]};
GenSoln={\[Sigma]\[CapitalPhi],-((\[Sigma]\[CapitalPhi] Sqrt[sd^2+\[Mu]\[CapitalPhi] \[Omega]d^2])/Sqrt[\[Mu]\[CapitalPhi]]),
\[Sigma]Z/Sqrt[sd^2+\[Omega]d^2]+(I \[Sigma]A\[Omega] \[Omega]d)/Sqrt[sd^2+\[Omega]d^2]+(\[Sigma]\[CapitalPhi] Sqrt[sd^2+\[Mu]\[CapitalPhi] \[Omega]d^2])/(sd Sqrt[\[Mu]\[CapitalPhi]]),
-\[Sigma]Z-I \[Sigma]A\[Omega] \[Omega]d-(\[Sigma]\[CapitalPhi] (sd^2+\[Mu]\[CapitalPhi] \[Omega]d^2))/(sd \[Mu]\[CapitalPhi]),
\[Sigma]A\[Omega]-(I \[Sigma]\[CapitalPhi] \[Omega]d)/sd,
-\[Sigma]A\[Omega] Sqrt[sd^2+\[Omega]d^2]+(I \[Sigma]\[CapitalPhi] \[Omega]d Sqrt[sd^2+\[Mu]\[CapitalPhi] \[Omega]d^2])/(sd Sqrt[\[Mu]\[CapitalPhi]]),
\[Sigma]A\[Nu],
-\[Sigma]A\[Nu] Sqrt[sd^2+\[Omega]d^2]
};

GenSolnRule=Table[uold[[i]]->GenSoln[[i]],{i,8}];
